<CsoundSynthesizer>
<CsOptions>
-odac -dm3 --sample-accurate
</CsOptions>
<CsInstruments>

sr      = 44100
ksmps   = 100
nchnls  = 2

    lua_opdef   "chord", {{
print (package.path)
package.path = package.path .. ";/storage/emulated/0/silencio/?.lua"
local ffi = require("ffi")
local math = require("math")
local string = require("string")
local Silencio = require ("Silencio")
local ChordSpace = require ("ChordSpace")
local csoundApi = ffi.load("csoundandroid")
ffi.cdef[[
    int csoundGetKsmps(void *);
    float csoundGetSr(void *);
    int csoundInputMessage(void *, const char *message);
    int csoundMessage(void *, const char *format, ...);
    struct chord_t {
      float *noteout;
      float *notein;
      float *kin;
      float *qin;
      float *tin;
      float *iin;
      float kold;
      float qold;
      float told;
      float iold;
    };
]]

local chord_ct = ffi.typeof('struct chord_t *')

local chord = Chord:new{0,4,7,10}
csoundApi.csoundMessage(csound, "chord: %s\\n", chord:information())
local modality = chord:clone()

function chord_init(csound, opcode, carguments)
    local p = ffi.cast(chord_ct, carguments)
    local pitch = p.notein[0]
    p.noteout[0] = pitch
    return 0
end

function chord_kontrol(csound, opcode, carguments)
    local p = ffi.cast(chord_ct, carguments)
    local pitch = p.notein[0]
    if p.kin[0] ~= p.kold then
        p.kold = p.kin[0] 
        if p.kin[0] == 1.0 then
            chord = chord:K()
            csoundApi.csoundMessage(csound, "kin: % 9.4f\\nchord: %s\\n", p.kin[0], chord:information())
        end
    end
    if p.qin[0] ~= p.qold then
        p.qold = p.qin[0] 
        if p.qin[0] == 1.0 then
            chord = chord:Q(1.0, modality)
            csoundApi.csoundMessage(csound, "qin: % 9.4f\\nchord: %s\\n", p.qin[0], chord:information())
        end
    end
    if p.tin[0] ~= p.told then
        p.told = p.tin[0] 
        if p.tin[0] == 1.0 then
            chord = chord:T(5.0)
            csoundApi.csoundMessage(csound, "tin: % 9.4f\\nchord: %s\\n", p.tin[0], chord:information())
        end
    end
    if p.iin[0] ~= p.iold then
        p.iold = p.iin[0] 
        if p.iin[0] == 1.0 then
            chord = chord:I(0.0)
            csoundApi.csoundMessage(csound, "iin: % 9.4f\\nchord: %s\\n", p.iin[0], chord:information())
        end
    end
    pitch = ChordSpace.conformPitchToChord(pitch, chord)
    p.noteout[0] = pitch
    return 0
end

}}

gasendL		init		0
gasendR		init		0 

            instr       FM

            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            ; Simple FM instrument.
            ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            print       p1, p2, p3, p4, p5,  p6
khz         =           cpsmidinn(p4)
kamplitude  =           ampdb(p5) * 0.1
ktrackpady  chnget      "slider1"
kcarrier    =           1 + 10 * ktrackpady
ktrackpadx  chnget      "slider2"
kmodulator  =           1 + 10 * ktrackpadx
iattack     =           0.03
irelease    =           0.1
isustain    =           p3
p3          =           iattack + isustain + irelease
            ; Intensity sidebands.
intensity   chnget      "slider3"
aindex      transeg     intensity * 20, p3, -5, 0.25
isine       ftgenonce   1, 0, 16384, 10, 1
asignal     foscili     kamplitude, khz, kcarrier, kmodulator, k(aindex), isine
adamping    linseg      0, iattack, 1, isustain, 1, irelease, 0
asignal     =           asignal * adamping * aindex
gasendL     =           gasendL + asignal
gasendR     =           gasendR + asignal
            endin

		instr		reverb
print p1, p2, p3
kreverbdelay   init     .8
kreverbdelay_  chnget      "trackpad.x"
kreverbdelay   =       (kreverbdelay_ > 0? kreverbdelay_: kreverbdelay)
aL,aR		reverbsc	gasendL, gasendR,kreverbdelay, 12000
		outs		aL, aR
		clear		gasendL, gasendR
		endin

            instr       logistic
keyin       init        0
key         init        0
kk          init        0
kq          init        0
kt          init        0
ki          init        0
kc          init        .89
ky          init        .5
ky1         init        .5
ktempo      init        4
kgross      chnget      "slider4"
kgross      scale       kgross, .75, .999
kfine       chnget      "slider5"
kk          chnget      "butt1"
kq          chnget      "butt2"
kt          chnget      "butt3"
ki          chnget      "butt4"
kfine       scale       kfine, 0, .001
kc          =           kgross + kfine
ktimes      times
ktrigger    metro       ktempo
            if ktrigger == 1 then
ky1         = ky
ky          = kc * ky1 * (1 - ky1) * 4
keyin       = floor(36 + 60 * ky)
            lua_ikopcall "chord", key, keyin,kk, kq, kt, ki
            event "i", 1, 0, 1/ktempo*4, key, 60
            endif
            printf      "nps:%9.4f t:%9.4f c:%12.8f y:%12.8f k:%4d\n", ktrigger, ktempo, ktimes, kc, ky, key

            
            endin
            
alwayson "reverb"
alwayson "logistic"

</CsInstruments>
<CsScore>
f 0 3600
</CsScore>
</CsoundSynthesizer>
